"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator.throw(value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const Rx = require("rxjs/Rx");
const BaseEntityManager_1 = require("./BaseEntityManager");
/**
 * Entity manager supposed to work with any entity, automatically find its repository and call its method, whatever
 * entity type are you passing. This version of ReactiveEntityManager works with reactive streams and observables.
 */
class ReactiveEntityManager extends BaseEntityManager_1.BaseEntityManager {
    // -------------------------------------------------------------------------
    // Constructor
    // -------------------------------------------------------------------------
    constructor(connection) {
        super(connection);
    }
    persist(targetOrEntity, maybeEntity) {
        // todo: extra casting is used strange tsc error here, check later maybe typescript bug
        const target = arguments.length === 2 ? targetOrEntity : targetOrEntity.constructor;
        const entity = arguments.length === 2 ? maybeEntity : targetOrEntity;
        return this.getReactiveRepository(target).persist(entity);
    }
    remove(targetOrEntity, maybeEntity) {
        // todo: extra casting is used strange tsc error here, check later maybe typescript bug
        const target = arguments.length === 2 ? targetOrEntity : targetOrEntity.constructor;
        const entity = arguments.length === 2 ? maybeEntity : targetOrEntity;
        return this.getReactiveRepository(target).remove(entity);
    }
    /**
     * Finds entities that match given conditions.
     */
    find(entityClass, conditionsOrFindOptions, options) {
        if (conditionsOrFindOptions && options) {
            return this.getReactiveRepository(entityClass).find(conditionsOrFindOptions, options);
        }
        else if (conditionsOrFindOptions) {
            return this.getReactiveRepository(entityClass).find(conditionsOrFindOptions);
        }
        else {
            return this.getReactiveRepository(entityClass).find();
        }
    }
    /**
     * Finds entities that match given conditions.
     */
    findAndCount(entityClass, conditionsOrFindOptions, options) {
        if (conditionsOrFindOptions && options) {
            return this.getReactiveRepository(entityClass).findAndCount(conditionsOrFindOptions, options);
        }
        else if (conditionsOrFindOptions) {
            return this.getReactiveRepository(entityClass).findAndCount(conditionsOrFindOptions);
        }
        else {
            return this.getReactiveRepository(entityClass).findAndCount();
        }
    }
    /**
     * Finds first entity that matches given conditions.
     */
    findOne(entityClass, conditionsOrFindOptions, options) {
        if (conditionsOrFindOptions && options) {
            return this.getReactiveRepository(entityClass).findOne(conditionsOrFindOptions, options);
        }
        else if (conditionsOrFindOptions) {
            return this.getReactiveRepository(entityClass).findOne(conditionsOrFindOptions);
        }
        else {
            return this.getReactiveRepository(entityClass).findOne();
        }
    }
    /**
     * Finds entity with given id.
     */
    findOneById(entityClass, id, options) {
        return this.getReactiveRepository(entityClass).findOneById(id, options);
    }
    /**
     * Executes raw SQL query and returns raw database results.
     */
    query(query) {
        const promiseFn = () => __awaiter(this, void 0, void 0, function* () {
            const queryRunner = yield this.connection.driver.createQueryRunner();
            const result = yield queryRunner.query(query);
            yield queryRunner.release();
            return Promise.resolve(result);
        });
        return Rx.Observable.fromPromise(promiseFn);
    }
    /**
     * Wraps given function execution (and all operations made there) in a transaction.
     */
    transaction(runInTransaction) {
        const promiseFn = () => __awaiter(this, void 0, void 0, function* () {
            const queryRunner = yield this.connection.driver.createQueryRunner();
            try {
                yield queryRunner.beginTransaction();
                const result = yield runInTransaction();
                yield queryRunner.commitTransaction();
                yield queryRunner.release();
                return Promise.resolve(result);
            }
            catch (err) {
                yield queryRunner.rollbackTransaction();
                yield queryRunner.release();
                throw err;
            }
        });
        return Rx.Observable.fromPromise(promiseFn);
    }
}
exports.ReactiveEntityManager = ReactiveEntityManager;

//# sourceMappingURL=ReactiveEntityManager.js.map
