"use strict";
const ColumnSchema_1 = require("./ColumnSchema");
class TableSchema {
    constructor(name) {
        this.columns = [];
        this.indices = [];
        this.foreignKeys = [];
        this.uniqueKeys = [];
        this.name = name;
    }
    removeColumn(column) {
        const index = this.columns.indexOf(column);
        if (index !== -1)
            this.columns.splice(index, 1);
    }
    removeIndex(indexSchema) {
        const index = this.indices.indexOf(indexSchema);
        if (index !== -1)
            this.indices.splice(index, 1);
    }
    removeForeignKey(foreignKey) {
        const index = this.foreignKeys.indexOf(foreignKey);
        if (index !== -1)
            this.foreignKeys.splice(index, 1);
    }
    removeUniqueByName(name) {
        const uniqueKey = this.uniqueKeys.find(uniqueKey => uniqueKey.name === name);
        if (!uniqueKey)
            return;
        const index = this.uniqueKeys.indexOf(uniqueKey);
        if (index === -1)
            return;
        this.uniqueKeys.splice(index, 1);
    }
    removePrimaryKey() {
        this.primaryKey = undefined;
    }
    static createFromMetadata(queryRunner, tableMetadata, columnMetadatas) {
        const tableSchema = new TableSchema(tableMetadata.name);
        tableSchema.columns = columnMetadatas.map(columnMetadata => ColumnSchema_1.ColumnSchema.create(queryRunner, columnMetadata));
        return tableSchema;
    }
    findChangedColumns(queryRunner, columnMetadatas) {
        return this.columns.filter(columnSchema => {
            const columnMetadata = columnMetadatas.find(columnMetadata => columnMetadata.name === columnSchema.name);
            if (!columnMetadata)
                return false; // we don't need new columns, we only need exist and changed
            return columnSchema.name !== columnMetadata.name ||
                columnSchema.type !== queryRunner.normalizeType(columnMetadata) ||
                columnSchema.comment !== columnMetadata.comment ||
                columnSchema.default !== columnMetadata.default ||
                columnSchema.isNullable !== columnMetadata.isNullable ||
                // columnSchema.isPrimary !== columnMetadata.isPrimary ||
                columnSchema.isGenerated !== columnMetadata.isGenerated;
        });
    }
}
exports.TableSchema = TableSchema;

//# sourceMappingURL=TableSchema.js.map
